<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Util\Messages;
use Util\Utilities;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class ContractedProductLine
{
    const OPEN_BILLING = 'FA';
    const OPEN_BILLING_FILTER = '%FA%';
    const CONTROL_FORM_PRODUCT = 'product';
    
    /** @var EntityManager $entityManager */
    protected $entityManager;
    /** @var \Util\Utilities $utilities */
    protected $utilities;
    /** @var \Teknisa\Model\Teknisa\Repositories\Contratolinp  $contratolinpRepository */
    protected $contratolinpRepository;
    /** @var \Teknisa\Model\Teknisa\Repositories\Sistcontlinp  $sistcontlinpRepository */
    protected $sistcontlinpRepository;
    /** @var \Teknisa\Model\Teknisa\Repositories\Licencauso  $licencausoRepository */
    protected $licencausoRepository;
    
    public function __construct(EntityManager $entityManager, Utilities $utilities)
    {
        $this->entityManager = $entityManager;
        $this->utilities = $utilities;
        $this->contratolinpRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Contratolinp');
        $this->sistcontlinpRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Sistcontlinp');
        $this->licencausoRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Licencauso');
    }
    
    public function findBillingReferringToMonth()
    {
        $clientId = Utilities::getEnvironment()->getCurrentCdCliente();
        return $this->contratolinpRepository->findBillingReferringToMonth($clientId);
    }
    
    public function findContractedProductLine($billingType, $billingMonth)
    {
        $clientId = Utilities::getEnvironment()->getCurrentCdCliente();
        if(!$billingType || $billingType == self::OPEN_BILLING_FILTER || $billingType == self::OPEN_BILLING){
            return $this->contratolinpRepository->findOpenBillings($clientId);
        } else {
            return $this->contratolinpRepository->findMonthBilling($billingMonth, $clientId);
        }
    }

    public function verifyProductDiscount()
    {
        $clientId = Utilities::getEnvironment()->getCurrentCdCliente();
        $contrato = false;
        $desconto = false;
        $descontoManutencao = false;
        $validations = [];
        $result = $this->contratolinpRepository->verifyProductDiscount($clientId);
        foreach($result as $contratolinp){
            if(!empty($contratolinp["NRSEQCONTRATO"])){
                $contrato = true;
            }
            if($contratolinp["VRDESCMANUTENCAO"] > 0){
                $descontoManutencao = true;
            }
            if($contratolinp["VRDESCLICENCAUSO"] > 0){
                $desconto = true;
            }
        }
        $validations["Contrato"] = $contrato;
        $validations["Desconto"] = $desconto;
        $validations["DescontoManutencao"] = $descontoManutencao;
        return $validations;
    }
    
    public function findSystems($productLineId, $billingType, $billingMonth, $conditions)
    {
        $clientId = Utilities::getEnvironment()->getCurrentCdCliente();
        if($billingType == self::OPEN_BILLING){
            return $this->sistcontlinpRepository->findSystemOpenBilling($clientId, $productLineId, $conditions);
        } else {
            return $this->sistcontlinpRepository->findSystemReferringToMonth($clientId, $productLineId, $billingMonth, $conditions);
        }
    }
    
    public function findUseLicenses($productLineId, $billingType, $billingMonth, $controlForm, $filters, $orderBy = array())
    {
        $clientId = Utilities::getEnvironment()->getCurrentCdCliente();
        $filters = $this->setFiltersUserLicenses($controlForm, $filters);
        $conditions     = DevOpsUtilities::createQueryFiltered($filters);
        $orderByString = '';
        foreach($orderBy as $idx => $order) {
            if($idx === 'DTLIBEPERSLUSO' || $idx === 'DTVALPERS'){
                $orderByString .= ' ORDER BY TO_DATE(' . $idx . ') ' . $order . ' NULLS LAST';
            } else {
                $orderByString .= ' ORDER BY ' . $idx . ' ' . $order;
            }
        }
        if($billingType == self::OPEN_BILLING){
            if($controlForm == self::CONTROL_FORM_PRODUCT){
                $dataset = $this->licencausoRepository->findUseLicenseOpenBillingsProduct($clientId, $productLineId, $conditions, $orderByString);
            }else{
                $dataset = $this->licencausoRepository->findUseLicenseOpenBillingsAccessPoint($clientId, $productLineId, $conditions, $orderByString);
            }

        } else {
            if($controlForm == self::CONTROL_FORM_PRODUCT){
                $dataset = $this->licencausoRepository->findUseLicenseReferringToMonthProduct($clientId, $productLineId, $billingMonth, $conditions, $orderByString);
            }else{
                $dataset = $this->licencausoRepository->findUseLicenseReferringToMonthAccessPoint($clientId, $productLineId, $billingMonth, $conditions, $orderByString);
            }
        }
        foreach($dataset as &$data){
            if(is_null($data['DTLIBEPERSLUSO'])){
                $data['DTLIBEPERSLUSO'] = '';
            }
            if(is_null($data['DTVALPERS'])){
                $data['DTVALPERS'] = '';
            }
        }
        return $dataset;
    }

    public function setFiltersUserLicenses($controlForm, $filters)
    {
        if ($controlForm != self::CONTROL_FORM_PRODUCT) {
            foreach ($filters as &$filter) {
                if ($filter['columnName'] == 'ATIVA' || $filter['columnName'] == 'VALIDA') {
                    if ($filter['value'] == 'Sim') {
                        $filter['value'] = 1;
                        $filter['operator'] = '>=';
                    } else {
                        $filter['value'] = 1;
                        $filter['operator'] = '<';
                    }
                }
            }
        }
        return $filters;
    }

}